@@ -60,6 +60,7 @@ struct StreamConfig {
     global_orientation: frame::Orientation,
     /// Orientation from a stream scope tag
     stream_orientation: Option<frame::Orientation>,
+    has_overlay: bool,
 }
 
 impl Default for StreamConfig {
@@ -68,6 +69,7 @@ impl Default for StreamConfig {
             info: None,
             global_orientation: frame::Orientation::Rotate0,
             stream_orientation: None,
+            has_overlay: false,
         }
     }
 }
@@ -84,10 +86,21 @@ pub struct PaintableSink {
     window_resized: AtomicBool,
 }
 
+#[derive(Debug, Default, glib::Enum, PartialEq, Eq, Copy, Clone)]
+#[repr(C)]
+#[enum_type(name = "GstGtk4PaintableSinkReconfigureMode")]
+pub enum ReconfigureMode {
+    Never,
+    Always,
+    #[default]
+    OverlayOnly,
+}
+
 #[derive(Default)]
 struct Settings {
     window_width: u32,
     window_height: u32,
+    reconfigure_on_window_resize: ReconfigureMode,
 }
 
 impl Drop for PaintableSink {
@@ -126,6 +139,11 @@ impl ObjectImpl for PaintableSink {
                     .blurb("the height of the main widget rendering the paintable")
                     .mutable_playing()
                     .build(),
+                glib::ParamSpecEnum::builder::<ReconfigureMode>("reconfigure-on-window-resize")
+                    .nick("Reconfigure on window resize")
+                    .blurb("send reconfigure event to upstream elements whenever window-width or window-height changes")
+                    .mutable_playing()
+                    .build(),
             ]
         });
 
@@ -193,6 +211,10 @@ impl ObjectImpl for PaintableSink {
                 let settings = self.settings.lock().unwrap();
                 settings.window_height.to_value()
             }
+            "reconfigure-on-window-resize" => {
+                let settings = self.settings.lock().unwrap();
+                settings.reconfigure_on_window_resize.to_value()
+            }
             _ => unimplemented!(),
         }
     }
@@ -215,6 +237,11 @@ impl ObjectImpl for PaintableSink {
                 }
                 settings.window_height = value;
             }
+            "reconfigure-on-window-resize" => {
+                let mut settings = self.settings.lock().unwrap();
+                let value = value.get().expect("type checked upstream");
+                settings.reconfigure_on_window_resize = value;
+            }
             _ => unimplemented!(),
         }
     }
@@ -340,6 +367,34 @@ impl ElementImpl for PaintableSink {
                             gst_video::VideoFormat::Y42b,
                             #[cfg(feature = "gtk_v4_20")]
                             gst_video::VideoFormat::Y444,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::I42010le,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::I42010be,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::I42210le,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::I42210be,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::Y44410le,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::Y44410be,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::I42012le,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::I42012be,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::I42212le,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::I42212be,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::Y44412le,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::Y44412be,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::Y44416le,
+                            #[cfg(feature = "gtk_v4_20")]
+                            gst_video::VideoFormat::Y44416be,
                             #[cfg(feature = "gtk_v4_14")]
                             gst_video::VideoFormat::Bgrx,
                             #[cfg(feature = "gtk_v4_14")]
@@ -556,7 +611,12 @@ impl BaseSinkImpl for PaintableSink {
                 .into(),
         };
 
-        self.config.lock().unwrap().info = Some(video_info);
+        let mut config = self.config.lock().unwrap();
+        config.info = Some(video_info);
+        config.has_overlay = caps
+            .features(0)
+            .unwrap()
+            .contains(gst_video::CAPS_FEATURE_META_GST_VIDEO_OVERLAY_COMPOSITION);
 
         Ok(())
     }
@@ -680,8 +740,19 @@ impl VideoSinkImpl for PaintableSink {
     fn show_frame(&self, buffer: &gst::Buffer) -> Result<gst::FlowSuccess, gst::FlowError> {
         gst::trace!(CAT, imp = self, "Rendering buffer {:?}", buffer);
 
-        if self.window_resized.swap(false, atomic::Ordering::SeqCst) {
-            gst::debug!(CAT, imp = self, "Window size changed, needs to reconfigure");
+        let reconfigure = {
+            if self.window_resized.swap(false, atomic::Ordering::SeqCst) {
+                let reconfigure_on_window_resize =
+                    self.settings.lock().unwrap().reconfigure_on_window_resize;
+                reconfigure_on_window_resize == ReconfigureMode::Always
+                    || (reconfigure_on_window_resize == ReconfigureMode::OverlayOnly
+                        && self.config.lock().unwrap().has_overlay)
+            } else {
+                false
+            }
+        };
+        if reconfigure {
+            gst::info!(CAT, imp = self, "Window size changed, needs to reconfigure");
             let obj = self.obj();
             let sink = obj.sink_pad();
             sink.push_event(gst::event::Reconfigure::builder().build());
